#!/usr/bin/perl -w
#
# GTKSubEditor
#
# gss_interface.pl
# Creates interface
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#
use strict;

sub createAbout
  {
    my $copy="GTKSubEditor v$main::version\n\n";
    $copy .="Copyright (c) 2005 Joni Valtanen\n";
    $copy .="GNU General Public Licence v2";

    # to the about window
    my $about_window;
    my $about_button;
    my $about_text;
    my $about_table;

    # creates about window
    $about_window = new Gtk2::Window("toplevel");
    $about_table = new Gtk2::Table(2,2);
    $about_button = new Gtk2::Button("OK");
    $about_text = new Gtk2::Label($copy);

    $about_window->set_title("About GTKSubEditor $main::version");
    $about_window->set_default_size(250,70);
    $about_window->signal_connect( "delete_event", sub { $about_window->destroy();} );
    $about_button->signal_connect( "clicked", \&closeWindow, $about_window );

    $about_table->attach_defaults( $about_text, 0, 1 , 0, 1 );
    $about_table->attach_defaults( $about_button, 0, 1 , 1, 2 );
    $about_table->set_border_width(4);

    $about_button->set_border_width(4);

    $about_window->set_modal( 1 );
    $about_window->add($about_table);

    $about_window->show_all();
    return 0;
  }

# Creates List (list_all) to scrolled window
sub createList
  {
    my ($button_all_apply) = @_;

    my $renderer;
    my $model_all;
    my $list_all;
    my @cell_data;

    $model_all = Gtk2::ListStore->new ('Glib::Uint', 'Glib::Uint', 'Glib::String', 'Glib::Boolean');
    $list_all = Gtk2::TreeView->new ($model_all);
    $list_all->set_rules_hint (TRUE);

    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 0);

    $cell_data[0]=$model_all;
    $cell_data[1]=$button_all_apply;

    $renderer->signal_connect (edited => \&cell_edited, \@cell_data);

    $list_all->insert_column_with_attributes (-1, "Start",
						      $renderer,
						      text => 0,
						      editable => 3
						     );
    # column for severities
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 1);
    $renderer->signal_connect (edited => \&cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1,"Stop",
						      $renderer,
						      text => 1,
						      editable => 3
						     );
    # column for description
    $renderer = Gtk2::CellRendererText->new;
    $renderer->set_data (column => 2);
    $renderer->signal_connect (edited => \&cell_edited, \@cell_data);
    $list_all->insert_column_with_attributes (-1,"Subtitle",
						      $renderer,
						      text => 2,
						      editable => 3
						     );

    return( $list_all, $model_all, $renderer );
  }


#Creates main window
sub createGUI
  {

    # main window
    my $window;

    # buttons Shift
    my $button_shift_shift;
    my $button_shift_start;
    my $button_shift_end;

    # buttons Convert
    my $button_convert_convert;

    #spins Convert
    my $adj_convert_ofps;
    my $adj_convert_nfps;
    my $spin_convert_ofps;
    my $spin_convert_nfps;

    my $label_all_filename;

    my $button_all_apply;

    #adjusters
    my $adj_shift_time;
    my $adj_shift_start;
    my $adj_shift_end;

#    $list_all = 0;

    my $notebook;
    my $frame_all_subfile;

    my $table_main;
    my $table_filename;
    my $table_split;
    my $table_join;
    my $table_convert;
    my $table_shift;
    my $table_all;

    my $button_quit;

    #optionmenu Convert
    my $option_ofps;
    my $option_nfps;
    my $option_ofps_menu;
    my $option_nfps_menu;

    my $ofps29_item;
    my $ofps25_item;
    my $ofps23_item;

    my $nfps29_item;
    my $nfps25_item;
    my $nfps23_item;


    #menus
    my $menu_bar;
    my $menuitem_file;
    my $menuitem_edit;
    my $menuitem_help;

    my $menu_file;
    my $menuitem_file_import;
    my $menuitem_file_reread;
    my $menuitem_file_saveas;
    my $menuitem_file_open;
    my $menuitem_file_quit;

    my $menu_edit;
    my $menuitem_edit_join;
    my $menuitem_edit_prefs;

    my $menu_help;
    my $menuitem_help_about;

    #labels
    my $label_notebook_shift;
    my $label_notebook_convert;
    my $label_notebook_split;

    my $label_shift_start;
    my $label_shift_end;
    my $label_shift_time;

    my $label_convert_ofps;
    my $label_convert_nfps;

    # Shift
    my $spin_shift_time;
    my $spin_shift_start;
    my $spin_shift_end;

    my $label_shift_sframes;
    my $label_shift_eframes;
    my $label_shift_tframes;

    # Split
    my $label_split_second;
    my $spin_split_second;
    my $adj_split_second;

    my $label_split_secondstarts;
    my $spin_split_secondstarts;
    my $adj_split_secondstarts;

    my $button_split_second;
    my $button_split_begin;
    my $button_split_end;


    #Scolled window & list
    my $scrolled_window_all;
    my $model_all;
    my $list_all;
    my $renderer_all;

    #other
    my $false = 0;
    my $true = 1;

    #datas for callbacks
    my @s_data; # shifting data
    my @ts_data; # toStart data
    my @te_data; # toEnd data
    my @cfps_data; # convertFPS data
    my @apply_data;
    my @ss_data; # split to start data
    my @split_data; # split data

    my @f_data;  #file dialogs data

    my @ofps25_data;
    my @ofps23_data;
    my @ofps29_data;

    my @nfps25_data;
    my @nfps23_data;
    my @nfps29_data;

    $window = new Gtk2::Window( "toplevel" );
    $window->set_default_size(400,400);


    # basic stings
    $frame_all_subfile = new Gtk2::Frame( "Subtitle file" );

    $table_main = new Gtk2::Table( 4, 2, $true );
    $table_filename = new Gtk2::Table( 1, 1, $true );
    $table_shift = new Gtk2::Table( 5, 4, $true );
    $table_convert = new Gtk2::Table( 3, 3, $true );
    $table_all = new Gtk2::Table( 2, 4, $true );

    $table_split = new Gtk2::Table( 5, 4, $true );
    $table_join = new Gtk2::Table( 5, 4, $true );

	
    #notebook
    $notebook = new Gtk2::Notebook();

    # Shift buttons
    $button_shift_shift = new Gtk2::Button( "Shift" );
    $button_shift_start = new Gtk2::Button( "From selection" );
    $button_shift_end = new Gtk2::Button( "From selection" );
    $button_quit = new Gtk2::Button( "Quit" );

    # Split buttons
    $button_split_second = new Gtk2::Button( "From selection" );
    $button_split_begin = new Gtk2::Button( "Save begin" );
    $button_split_end = new Gtk2::Button( "Save end" );

    # Convert buttons
    $button_convert_convert = new Gtk2::Button( "Convert FPS" );

    # Convert option menus
    $option_ofps = new Gtk2::OptionMenu();
    $option_nfps = new Gtk2::OptionMenu();
	
    # menus
    $menu_bar = new Gtk2::MenuBar();
    $menuitem_file = new Gtk2::MenuItem( "_File" );
    $menuitem_edit = new Gtk2::MenuItem( "_Edit" );
    $menuitem_help = new Gtk2::MenuItem( "_Help" );

    $menu_file = new Gtk2::Menu();
    $menuitem_file_open = new Gtk2::MenuItem( "_Open" );
    $menuitem_file_import = new Gtk2::MenuItem( "_Import" );
    $menuitem_file_reread = new Gtk2::MenuItem( "_Reload..." );
    $menuitem_file_saveas = new Gtk2::MenuItem( "Save _As" );
    $menuitem_file_quit = new Gtk2::MenuItem( "_Quit" );

    $menu_edit = new Gtk2::Menu();
    $menuitem_edit_join = new Gtk2::MenuItem( "_Join After" );
    $menuitem_edit_prefs = new Gtk2::MenuItem( "_Prefs" );

    $menu_help = new Gtk2::Menu();
    $menuitem_help_about = new Gtk2::MenuItem( "_About" );

    # option menus Convert
    $option_ofps_menu = new Gtk2::Menu();
    $option_nfps_menu = new Gtk2::Menu();

    $ofps23_item = new Gtk2::MenuItem( "23.976" );
    $ofps25_item = new Gtk2::MenuItem( "25" );
    $ofps29_item = new Gtk2::MenuItem( "29" );

    $nfps23_item = new Gtk2::MenuItem( "23.976" );
    $nfps25_item = new Gtk2::MenuItem( "25" );
    $nfps29_item = new Gtk2::MenuItem( "29" );

    $option_ofps_menu->append( $ofps23_item );
    $option_ofps_menu->append( $ofps25_item );
    $option_ofps_menu->append( $ofps29_item );

    $option_nfps_menu->append( $nfps23_item );
    $option_nfps_menu->append( $nfps25_item );
    $option_nfps_menu->append( $nfps29_item );


    # labels
    $label_notebook_shift = new Gtk2::Label("Shift");
    $label_notebook_convert = new Gtk2::Label("Convert");
    $label_notebook_split = new Gtk2::Label("Split");

    $label_shift_start = new Gtk2::Label("Start:");
    $label_shift_end = new Gtk2::Label("End:");
    $label_shift_time = new Gtk2::Label("Time:");



    #	$label_edit_fs = new Gtk2::Label("Start:");
    #	$label_edit_fe = new Gtk2::Label("End:");
    #	$label_edit_sub = new Gtk2::Label("Sub:");

    $label_shift_sframes = new Gtk2::Label("from frame");
    $label_shift_eframes = new Gtk2::Label("to frame");
    $label_shift_tframes = new Gtk2::Label("frames");


    $label_shift_start->set_justify( "left" );
    $label_shift_end->set_justify( "left" );
    $label_shift_time->set_justify( "left" );

    #Other labels
    $label_split_second = new Gtk2::Label("Split second subtitle from:");
    $label_split_secondstarts = new Gtk2::Label("Second subtitle starts:");

    $label_all_filename = new Gtk2::Label("Not loaded.");

    $label_convert_ofps = new Gtk2::Label("Original FPS:");
    $label_convert_nfps = new Gtk2::Label("New FPS:");
	
    #adjustments to spin bittons
    $adj_shift_time = new Gtk2::Adjustment( 0,
					    -999999,
					    999999,
					    1,
					    1,
					    10 );

    $adj_shift_start = new Gtk2::Adjustment( 0,
					     0,
					     999999,
					     1,
					     1,
					     10 );

    $adj_shift_end = new Gtk2::Adjustment( 0,
					   0,
					   999999,
					   1,
					   1,
					   10 );

    #Convert adjustments
    $adj_convert_ofps = new Gtk2::Adjustment( 0,
					    1.0,
					    100,
					    0.1,
					    1,
					    10 );

    $adj_convert_nfps = new Gtk2::Adjustment( 0,
					    1.0,
					    100,
					    0.1,
					    1,
					    10 );

    #Split adjustements
    $adj_split_second = new Gtk2::Adjustment( 0,
					   0,
					   999999,
					   1,
					   1,
					   10 );
    $adj_split_secondstarts = new Gtk2::Adjustment( 0,
					   1,
					   999999,
					   1,
					   1,
					   10 );

    # Shift spinbuttons
    $spin_shift_start = new Gtk2::SpinButton($adj_shift_start, 1.0, 0);
    $spin_shift_end = new Gtk2::SpinButton($adj_shift_end, 1.0, 0);
    $spin_shift_time = new Gtk2::SpinButton($adj_shift_time, 1.0, 0);

    # Convert spins
    $spin_convert_ofps = new Gtk2::SpinButton($adj_convert_ofps, 1.0, 3);
    $spin_convert_nfps = new Gtk2::SpinButton($adj_convert_nfps, 1.0, 3);
    $spin_convert_ofps->set_value( "23.976" );
    $spin_convert_nfps->set_value( "25.00" );

    # Split spins
    $spin_split_second = new Gtk2::SpinButton($adj_split_second, 1.0, 0);
    $spin_split_secondstarts = new Gtk2::SpinButton($adj_split_secondstarts, 1.0, 0);
    $spin_split_secondstarts->set_value( "1" );

    $button_all_apply = new Gtk2::Button( "Apply edited" );

    # list	
    $scrolled_window_all = new Gtk2::ScrolledWindow( undef, undef );
    $scrolled_window_all->set_policy( 'automatic', 'always' );
    $scrolled_window_all->show();

    #Create list
    ($list_all, $model_all, $renderer_all) = &createList($button_all_apply);

    # File menu
    $menu_file->append( $menuitem_file_open );
    $menu_file->append( $menuitem_file_import );
    $menu_file->append( $menuitem_file_reread );
    $menu_file->append( $menuitem_file_saveas );
    $menu_file->append( $menuitem_file_quit );

#    $menuitem_file->show();
    $menuitem_file->set_submenu( $menu_file );

    $menu_file->show();

    #show option menu
    $option_ofps->set_menu($option_ofps_menu);
    $option_nfps->set_menu($option_nfps_menu);

    $option_ofps->set_history( 0 );
    $option_nfps->set_history( 1 );

    #Edit menu
    $menu_edit->append( $menuitem_edit_join );
    $menu_edit->append( $menuitem_edit_prefs );
    $menuitem_edit->set_submenu( $menu_edit );

    # Help menu
    $menu_help->append( $menuitem_help_about );
    $menuitem_help->set_submenu( $menu_help );


    # build Convert option menu

    # build menu bar
    $menuitem_file_reread->set_sensitive(0);
    $menuitem_file_saveas->set_sensitive(0);

	
    #notebook
    $notebook->prepend_page( $table_split, $label_notebook_split);
    $notebook->prepend_page( $table_convert, $label_notebook_convert);
    $notebook->prepend_page( $table_shift, $label_notebook_shift );

    # table
    my $xoptions = "shrink";
    my $yoptions = "shrink";
    my $xpadding = 0;
    my $ypadding = 0;

    $scrolled_window_all->add( $list_all );

    # table shift
    $table_shift->set_homogeneous( $false);

    $table_shift->attach( $label_shift_start, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_start, 1, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_sframes, 2, 3, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $button_shift_start, 3, 4, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $label_shift_end, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_end, 1, 2, 1, 2, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_eframes, 2, 3, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $button_shift_end, 3, 4, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $label_shift_time, 0, 1, 2, 3, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $spin_shift_time, 1, 2, 2, 3, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->attach( $label_shift_tframes, 2, 3, 2, 3, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_shift->attach( $button_shift_shift, 0, 4, 3, 4, "fill", $yoptions, $xpadding, $ypadding );
    $table_shift->set_border_width(4);
    #	$frame_shift->add($table_shift);

    # table convert
    $table_convert->set_homogeneous( $false);
    $table_convert->attach( $label_convert_ofps, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $spin_convert_ofps, 1, 2, 0, 1, ["expand","fill"], $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $option_ofps, 2,3,0,1,["expand","fill"], $yoptions, $xpadding, $ypadding );

    $table_convert->attach( $label_convert_nfps, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $spin_convert_nfps, 1, 2, 1, 2,  "fill", $yoptions, $xpadding, $ypadding );
    $table_convert->attach( $option_nfps, 2,3,1,2,["expand","fill"], $yoptions, $xpadding, $ypadding );

    $table_convert->attach( $button_convert_convert, 0, 3, 2, 3, "fill", $yoptions, $xpadding, $ypadding );
    $table_convert->set_border_width(4);

    # table split
    $table_split->set_homogeneous( $false);
    $table_split->attach( $label_split_second, 0, 1, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_split->attach( $spin_split_second, 1, 2, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_split->attach( $button_split_second, 2, 3, 0, 1, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_split->attach( $label_split_secondstarts, 0, 1, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );
    $table_split->attach( $spin_split_secondstarts, 1, 2, 1, 2, $xoptions, $yoptions, $xpadding, $ypadding );

    $table_split->attach( $button_split_begin, 1, 3, 2, 3, ["expand","fill"], $yoptions, $xpadding, $ypadding );
    $table_split->attach( $button_split_end, 1, 3, 3, 4, ["expand","fill"], $yoptions, $xpadding, $ypadding );


    $table_split->set_border_width(4);


    # table all
    $table_all->set_homogeneous( $false );

    $frame_all_subfile->add( $label_all_filename ); #, 1, 2, 2, 3, "expand", $yoptions, $xpadding, $ypadding);
    $table_all->attach_defaults( $frame_all_subfile, 0, 2, 0, 1); #, "fill", $yoptions, $xpadding, $ypadding);
    $table_all->attach_defaults( $scrolled_window_all, 0, 2, 1, 3 );
    $table_all->attach( $button_all_apply, 0, 2, 3, 4, "fill", $yoptions, $xpadding, $ypadding );
    $table_all->attach( $button_quit, 1, 2, 4, 5, "fill", $yoptions, $xpadding, $ypadding );
    $table_all->set_border_width(4);

    # table main
    $table_main->set_homogeneous( $false );

    $table_main->attach( $menu_bar, 0, 2, 0, 1, "fill", $yoptions, $xpadding, $ypadding);
    $table_main->attach( $notebook, 0, 2, 1, 2, "fill", $yoptions, $xpadding, $ypadding);
    $table_main->attach_defaults( $table_all, 0, 2, 3, 4);
    #	$table_main->set_border_width(2);


    $menu_bar->append($menuitem_file);	
    $menu_bar->append($menuitem_edit);	
    $menu_bar->append($menuitem_help);	

    $button_shift_shift->set_sensitive(0);
    $button_convert_convert->set_sensitive(0);
    $button_all_apply->set_sensitive(0);

    if ($main::use_mplayer) {
      $menuitem_file_import->set_sensitive(1);
    } else {
      $menuitem_file_import->set_sensitive(0);
    }

    $menuitem_edit_join->set_sensitive(0);

    $f_data[0]=$main::filename;
    $f_data[1]=$model_all;
    $f_data[2]=$button_shift_shift;
    $f_data[3]=$button_convert_convert;
    $f_data[4]=$label_all_filename;
    $f_data[5]=$menuitem_edit_join;
    $f_data[6]=$table_split;

    # signal for menus
    $menuitem_file_open->signal_connect( 'activate', \&fileOpen, \@f_data );
    $menuitem_file_import->signal_connect( 'activate', \&fileImport, \@f_data );
    $menuitem_file_reread->signal_connect( 'activate', \&fileReread, \@f_data );
    $menuitem_file_saveas->signal_connect( 'activate', \&fileSaveAs );
    $menuitem_file_quit->signal_connect( 'activate', \&quitProgram, $window );

    $menuitem_edit_join->signal_connect( 'activate', \&editJoin, \@f_data );
    $menuitem_edit_prefs->signal_connect( 'activate', \&editPreferences, $menuitem_file_import );

    $menuitem_help_about->signal_connect( 'activate', \&createAbout );

    $ofps25_data[0]="25";
    $ofps25_data[1]=$spin_convert_ofps;
    $ofps23_data[0]="23.976";
    $ofps23_data[1]=$spin_convert_ofps;
    $ofps29_data[0]="29";
    $ofps29_data[1]=$spin_convert_ofps;

    $nfps25_data[0]="25";
    $nfps25_data[1]=$spin_convert_nfps;
    $nfps23_data[0]="23.976";
    $nfps23_data[1]=$spin_convert_nfps;
    $nfps29_data[0]="29";
    $nfps29_data[1]=$spin_convert_nfps;

    #signals to Convert option menu
    $ofps25_item->signal_connect( 'activate', \&FPS, \@ofps25_data );
    $ofps23_item->signal_connect( 'activate', \&FPS, \@ofps23_data );
    $ofps29_item->signal_connect( 'activate', \&FPS, \@ofps29_data );

    $nfps25_item->signal_connect( 'activate', \&FPS, \@nfps25_data );
    $nfps23_item->signal_connect( 'activate', \&FPS, \@nfps23_data );
    $nfps29_item->signal_connect( 'activate', \&FPS, \@nfps29_data );

    #Shifting callbackdata
    $s_data[0] = $adj_shift_time;
    $s_data[1] = $adj_shift_start;
    $s_data[2] = $adj_shift_end;
    $s_data[3] =  $menuitem_file_reread;
    $s_data[4] =  $menuitem_file_saveas;
    $s_data[5]=$model_all;
    $s_data[6]=$button_shift_shift;
    $s_data[7]=$button_convert_convert;

    #toStart callbackdata
    $ts_data[0] = $list_all;
    $ts_data[1] = $adj_shift_start;

    #toEnd callbackdata
    $te_data[0] = $list_all;
    $te_data[1] = $adj_shift_end;

    #convertFPS callbackdata
    $cfps_data[0] = $spin_convert_ofps;
    $cfps_data[1] = $spin_convert_nfps;
    $cfps_data[2] = $menuitem_file_reread;
    $cfps_data[3] = $menuitem_file_saveas;
    $cfps_data[4] =  $model_all;
    $cfps_data[5] =  $button_shift_shift;
    $cfps_data[6] =  $button_convert_convert;

    #applyChangesToFile callback data
    $apply_data[0] =  $model_all;
    $apply_data[1] =  $button_shift_shift;
    $apply_data[2] =  $button_convert_convert;
    $apply_data[3] =  $menuitem_file_saveas;
    $apply_data[4] =  $menuitem_file_reread;

    $ss_data[0] = $list_all; #list_all
    $ss_data[1] = $adj_split_second;

    $split_data[0] = $adj_split_second; #second starts
    $split_data[1] = $adj_split_secondstarts; #second starts
    $table_split->set_sensitive(0);

    # signals window
    $button_quit->signal_connect( "clicked", \&quitProgram, $window );

    $button_shift_start->signal_connect( "clicked", \&toStart, \@ts_data );
    $button_shift_end->signal_connect( "clicked", \&toEnd, \@te_data);
    $button_shift_shift->signal_connect( "clicked", \&shifting, \@s_data );

    $button_convert_convert->signal_connect( "clicked", \&convertFPS, \@cfps_data);

    $button_split_second->signal_connect( "clicked", \&toStart, \@ss_data );
    $button_split_begin->signal_connect( "clicked", \&splitSaveBegin, \@split_data);
    $button_split_end->signal_connect( "clicked", \&splitSaveEnd, \@split_data);

    $window->signal_connect( "delete_event" => \&delete_event );
    $window->signal_connect(destroy => sub { Gtk2->main_quit; });

    $button_all_apply->signal_connect( "clicked", \&applyEditSubtitle, \@apply_data);
	
    $window->set_border_width( 1 );
    $window->add( $table_main );

    $window->show_all();

#    $table_split->hide();
  }

1;
