#!/usr/bin/perl
#
# SetSubFPS v.0.1
# Converting Subtitlefiles to other framerate
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#
# Command line arguments:
# -i inputfile
# -o outputfile (optional)
# -f original framerate
# -n new frame rate
#
#
# Exit codes:
# 0 - out from help and version
# 1 - not enought arguments
# 2 - no inputfile
# 3 - time is not numeric value
#
#
# Version information
#
# 0.1  -  051102, Joni Valtanen
#    * created
#
use Getopt::Std;

$version = "0.1";

# prints out usage information
sub usage
  {
    print "\nSetSubFPS v$version\n\n";
    print "Usage: setsubfps.pl -f <original fps> -n <new framerate> -i <ifile>\n";
    print " Optional swiches:\n    [-o <ofile>]\n\n";
    print "  -f     original framerate\n";
    print "  -n     new framerate\n";
    print "  -i     filename to reading\n";
    print "  -o     filename to writing. If no name is given stdout is used.\n";
    print "  -h     prints this help\n";
    print "  -v     verbose mode\n";
    print "  -V     prints only version information\n\n";
  }

# is numerical value
# decimal numbers are not allowed eighter
sub isNumerical
  {
    ($test) = @_;
    if (!($test !~ m/\D/) ) {
      &usage();
      return 0;
    }
    return 1;
  }

# Get options
getopt('iofn', \%opts);

$input = $opts{'i'};
$output = $opts{'o'};
$nfps = $opts{'n'};
$fps = $opts{'f'};

if ($opts{'V'}) {
  print "\nSetSubFPS v$version\n\n";
  exit 0;
}

# help
if ($opts{'h'}) {
  &usage();
  exit 0;
}

# Some tests are arguments ok
if (!$nfps || !$fps || !$input ) {
  &usage;
  exit 1;
}

#if (!&isNumerical($time)) {
#  exit 3;
#}

#if (!$from) {
#  $from = 0;
#}
#if (!$end) {
#  $end = 999999;
#}

#print some information
if ($opts{'v'}) {
  print "Original FPS: $fps \n";
  print "New FPS: $fps \n";
  print "From file $input ";
  print "to file $output.\n\n";
}

open IFILE, "$input" or die("Couldn't open $input\n") ;

# if user gave output file
# set sdtout to print this
if ($output) {
  open STDOUT, ">$output" or die("Couldn't open $output\n") ;
}

$prop=$nfps/$fps;

# reads whole file line by line
# and writes it to stdout
while ($test =  readline(IFILE)) {
  $line = $test;
  @line_t1 = split(/}/, $line);
  @line_t2 = split(/{/, $line_t1[0]);
  @line_t3 = split(/{/, $line_t1[1]);

  # from
#  if ( ( $from <= $line_t2[1] ) && ( $end >= $line_t3[1] ) ) {
    $line_t2[1] = int( $line_t2[1] * $prop);
    $line_t3[1] = int( $line_t3[1] * $prop);
#  }



  # drops negatives and zeros out ;)
  if ( $line_t2[1] > 0 ) {
    print "{$line_t2[1]}";
    print "{$line_t3[1]}";
    print "$line_t1[2]";
  }
}

#close everything
close IFILE;

if ($output) {
  close STDOUT;
}


